#!/bin/sh

baselog="base.log"
checklog="check.log"
#report_file="report/report-`date`"
feature_test_case_path="test_case/feature/sysak/"
performace_test_case_path="test_case/performace/"
wait_done="false"
test_log="report/test_log"
report_file="report/report.txt"
tmpfile="report/report_tmp.log"

uninstall() {
    rpm -qa sysak |grep "sysak"
    if [ $? -eq 0 ];then
        rpm -e sysak
    fi
}

install(){
	uninstall
	rpm -ivh https://mirrors.openanolis.cn/sysak/packages/gate_check_in/sysak-2.2.0-1.debug.gate.x86_64.rpm
}

clean(){
	rm -rf $report_file
	rm -rf $test_log
	rm -rf $tmpfile
}

log(){
    echo "$1" >> $report_file
}

save_sys_info() {
	echo > $baselog

	echo "kernel log:" >> $baselog
	dmesg | tail -n 1 >> $baselog
	
	echo "cpuinfo:" >> $baselog
	mpstat $cpuarg 1 1 | awk 'END {print $3" "$5" "$6" "$7" "$8" "$12}' >> $baselog

	echo "meminfo:" >> $baselog
	cat /proc/meminfo | grep MemAvailable >> $baselog
}

check_exception() {
	kernel_log=`cat $baselog | grep "kernel log:" -A 1 | grep -v "kernel log:"`
	new_log=dmesg | grep "\$kernel_log" -A 99999 | grep -v "\$kernel_log"
	echo "kernel log:" >> $checklog
	echo $new_log >> $checklog
	#check if bug \warning\ error\ trace\ rcustall\ lockup etc
	echo $new_log | grep -E "BUG|WARNING|LOCKUP|stall|hung_task" > /dev/null
	if [ $? == 0 ]; then
		echo "kernel error happened, check failed"
	fi
}

check_cpu_cost() {
	cpu_log=`cat $baselog | grep "cpuinfo:" -A 1 | grep -v "cpuinfo:"`
	cpu_idle_before=`echo $cpu_log | awk '{print $6}'`

	new_cpu=`mpstat $cpuarg 1 1 | awk 'END {print $3" "$5" "$6" "$7" "$8" "$12}'`
	echo "cpuinfo:" >> $checklog
	echo $new_cpu >> $checklog
	
	cpu_idle=`echo $new_cpu | awk '{print $6}'`
	if [ $(echo "$cpu_idle + 100 < $cpu_idle_before" | bc) -eq 1 ] ;then
		echo "cpu cost too much, check failed"
	fi
}

check_mem_cost() {
	mem_log=`cat $baselog | grep "meminfo:" -A 1 | grep -v "meminfo:"`
	avail_mem_before=`echo $mem_log | awk '{print $2}'`
	
	new_meminfo=`cat /proc/meminfo | grep MemAvailable`
	echo "meminfo:" >> $checklog
	echo $new_meminfo >> $checklog

	avail_mem=`echo $new_meminfo | awk '{print $2}'`
	if [ $(echo "$avail_mem + 102400 < $avail_mem_before" | bc) -eq 1 ] ;then
		echo "mem cost too much, check failed"
	fi
}

tool_running_check() {
	echo "doing sysak tool check"
	save_sys_info
	if [ $wait_done = true ]; then
		echo "excute $1" 
		sh $1
	else
		echo "excute $1 backgroud"
		sh $1 &
	fi
	sleep 1
	echo > $checklog
	check_exception
	check_cpu_cost
	check_mem_cost
	sleep 5
}

run_feature_test(){
    echo "----$1----"
    for i in `ls $feature_test_case_path/$1`;do
		echo $i | grep txt
		if [ $? -eq 0 ];then
			continue
		fi
		echo "$i testing..."
		rm -rf $tmpfile
        tool_running_check $feature_test_case_path/$1/$i  >> $tmpfile
		cat $tmpfile >> $test_log
		cat $tmpfile |grep -i -E "fail|error"
        if [ $? -eq 0 ];then
            log "${i}: Fail"
        else
			log "${i}: Pass"
        fi
		echo "$i tested"
    done
}

run_performace_test(){
    echo  "----$1----"
    for i in `ls $performace_test_case_path/$1`;do
        tool_running_check $i
        if [ $? -ne 0 ];then
            log "${i}: Fail"
        elif [ $? -eq 0 ];then
        	log "${i}: Pass"
        fi
    done
}

main(){
	clean
    log "----Feature test----"
	wait_done="true"
    #run_feature_test "download-test"
	wait_done="false"
	#install
	run_feature_test "usage-test"
    run_feature_test "generic-test"
    run_feature_test "io-test"
    run_feature_test "mem-test"
    run_feature_test "net-test"
	run_feature_test "sched-test"
	run_feature_test "hw-test"
	run_feature_test "fs-test"
	run_feature_test "virt-test"
	run_feature_test "monitor-test"
    log "----Performace test----"
    #run_performace_test ""
}

main
