#!/bin/sh

#Usage: sysak workqlatency [options] [args]
#Options:
#    --threshold/-l		specify the threshold time(ms), default=10ms
#    --help/-h			help info
#example: sysak workqlatency -t 10  #trace work runtime and latency statistics
sysdata_file="/var/log/sysak/syshung_detector/sysdata"
temp_log_path="syshung_detector_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test syshung_detector failed." 
    clean
    exit 1
}

function test_help() {
    sysak syshung_detector -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: syshung_detector" > /dev/null
    if [ $? -ne 0 ];then
        handle_failed
    else
        echo "Test syshung_detector helpinfo success."
    fi
}

function test_check_hung() {
    sysak syshung_detector -c > $temp_log_path 2>&1 && cat $temp_log_path | grep "isHung" > /dev/null
    if [ $? -ne 0 ];then
        handle_failed
    else
        echo "Test sysak syshung_detector -c success."
    fi

    sysak syshung_detector -c -j > $temp_log_path 2>&1 && cat $temp_log_path | grep "isHung" > /dev/null
    if [ $? -ne 0 ];then
        handle_failed
    else
        echo "Test sysak syshung_detector -c -j success."
    fi
}

function test_collect_system_data() {
    sysak syshung_detector -c -d  > $temp_log_path 2>&1 && cat $temp_log_path | grep "command not found" > /dev/null
    if [ $? -ne 0 ];then
        echo "some tools not installed, test failed!"
        handle_failed
    fi

    cat $temp_log_path | grep  -E "mem data collect succeeded|sched data collect succeeded" > /dev/null
    if [ $? -ne 0 ];then
        handle_failed
    else
        echo "Test sysak syshung_detector -c -d success."
    fi
}

function test_collect_system_data_by_ebpf() {
    rm -rf $sysdata_file/*
    sysak syshung_detector -c -b

    ls $sysdata_file | grep bpf
    if [ $? -ne 0 ];then
        handle_failed
    else
        echo "Test sysak syshung_detector -c -b success."
    fi
}

function test_collect_task_data() {
    rm -rf $sysdata_file/*
    sysak syshung_detector -p 1 > $temp_log_path 2>&1 && cat $temp_log_path | grep -E "mem data collect succeeded|sched data collect succeeded" > /dev/null
    if [ $? -ne 0 ];then
        handle_failed
    else
        echo "Test sysak syshung_detector -p success."
    fi
}

function main() {
    test_help
    test_check_hung
    test_collect_system_data
    #test_collect_system_data_by_ebpf
    test_collect_task_data
    clean
}

main
