#!/bin/sh
#  Usage: sysak hw_event [-c container] [-s TIME]
#  Options:
#  -c container    container(docker) name or id, default all container 
#  -s TIME         specify how long to run, default 5s 
#eg. sysak hw_event -c pause -s 10

temp_log_path="hw_event.log"

function clean() {
    pkill hw_event
    docker stop hwcontainer
    docker rm hwcontainer -f
    rm $temp_log_path
}

function handle_failed() {
    echo "Test hw_event failed." 
    clean
    exit 1
}

function check_fail() {
    check_start=`cat $temp_log_path | grep "Failed to open and load BPF skeleton" | wc -l`
    check_syms_cache=`cat $temp_log_path | grep "Failed to load and verify BPF skeleton" | wc -l`
    check_bpf=`cat $temp_log_path | grep "failed to load kallsyms" | wc -l`
    a1=`cat $temp_log_path | grep "\\}" | wc -l`
    a2=`cat $temp_log_path | grep "\\{" | wc -l`
    if (test $check_start -ne 0) || (test $check_syms_cache -ne 0) || (test $check_bpf -ne 0) || [ "$a1" != "$a2" ]
    then
        handle_failed  
    fi    
}

function prepare_container() {
    docker run --name hwcontainer --net=host -itd busybox /bin/sh
    if test $? -ne 0
    then
        handle_failed
    fi
}

function test_help() {
    sysak -g hw_event --help > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: sysak hw_event" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test hw_event --help success."
    fi
}

function test_all() {
    sysak -g hw_event > $temp_log_path 2>&1 &
    sleep 1
    pkill hw_event
    sleep 1
    
    check_fail
    echo "Test hw_event success."
}

function test_t() {
    sysak -g hw_event -t 50 > $temp_log_path 2>&1 &
    sleep 1
    pkill hw_event
    sleep 1
    
    check_fail
    echo "Test hw_event -t success."
}

function test_c() {
    prepare_container
    sysak -g hw_event -c hwcontainer > $temp_log_path 2>&1 &
    sleep 1
    pkill hw_event
    sleep 1

    check_fail
    echo "Test hw_event -c success."
}

function test_s() {
    sysak -g hw_event -s 4 > $temp_log_path 2>&1 &
    sleep 5

    pid1=`pidof hw_event`
    if [ ! $pid1 ]; then
        echo "end"  
    else  
        echo $pid1
        echo "didnt end"  
        handle_failed
    fi 

    check_fail
    echo "Test hw_event -s success."
}

function main() {
    test_help
    test_all
    test_t
    #test_f
    test_c
    #test_S
    clean
}

main
