#!/bin/sh

#sysak numa_access: show numa access information
#options: -h help information
#         -p pid, specify the pid
#         -c cpu, specify the cpu
#         -i interval, the interval checking the numa access times
temp_log_path="numa_access_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test numa_access failed."
    clean
    exit 1
}

function test_help() {
    sysak numa_access -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "sysak numa_access: show numa access information" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test numa_access helpinfo success."
    fi
}

function test_numa_access() {
    sysak numa_access -p 1 > $temp_log_path 2>&1 && cat $temp_log_path | grep "pid" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak numa_access -p success."
    fi

    sysak numa_access -c 1 > $temp_log_path 2>&1 && cat $temp_log_path | grep "cpu" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak numa_access -c success."
    fi

    sysak numa_access -p 1 -c 1 > $temp_log_path 2>&1 && cat $temp_log_path | grep -E "cpu|pid" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak numa_access -p 1 -c 1 success."
    fi

    sysak numa_access -p 1 -c 1 > $temp_log_path 2>&1 && cat $temp_log_path | grep "Command fail" > /dev/null
    if test $? -eq 0
    then
        handle_failed
    else
        echo "Test fail log success."
    fi
}

function main() {
    test_help
    test_numa_access
    clean
}

main
