#!/bin/sh
#
# usage: ioMonitor.py [-h] [-y YAML_FILE] [-s SET_CFG] [-r] [-o]

# start ioMonitor.

# optional arguments:
#   -h, --help            show this help message and exit
#   -y YAML_FILE, --yaml_file YAML_FILE
#                         Specify the socket pipe for data upload and exception
#                         log path
#   -s SET_CFG, --set_cfg SET_CFG
#                         set monitor cfg, like -s 'xxxx=xxx,xxx=xxx' iowait,
#                         The min cpu-iowait not report exceptions(1~100,
#                         default 5). await, The min partition-await not report
#                         exceptions(N ms, default 10). util, The min partition-
#                         util not report exceptions(1~100, default 20). bps,
#                         The min partition-bps not report exceptions(bytes,
#                         default 30MB). iops, The min partition-iops not report
#                         exceptions(default 150). cycle, The cycle of
#                         Monitoring data collection(default 500ms). diagIowait,
#                         Disable or enable diagnose while reporting iowait
#                         exceptions(default on). diagIolat, Disable or enable
#                         diagnose while reporting latency exceptions(default
#                         on). diagIoburst, Disable or enable diagnose while
#                         reporting ioburst exceptions(default on). diagIohang,
#                         Disable or enable diagnose while reporting iohang
#                         exceptions(default on).
#   -r, --reset_cfg       Reset cfg to default
#   -o, --only_set_cfg    Only set cfg

# e.g.
#   ./ioMonitorMain.py -y [yaml_file]
#             Start ioMonitor
#   ./ioMonitorMain.py -y [yaml_file] --reset_cfg --only_set_cfg
#             Only reset cfg to default
#   ./ioMonitorMain.py -y [yaml_file] -s 'iowait=10,iops=200,diagIowait=on' --only_set_cfg
#             Only set min-iowait&&min-iops and disable iowait diagnose to config.

temp_log_path="ioMonitor_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test ioMonitor failed." 
    cat $temp_log_path
    clean
    exit 1
}

function test_help() {
    sysak ioMonitor -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "usage: ioMonitor" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test ioMonitor helpinfo success."
    fi
}

function test_ioMonitor() {
    unity_base_yaml_path=`rpm -ql sysak | grep base.yaml | grep "sysak_components"`
    sysak ioMonitor -y $unity_base_yaml_path > $temp_log_path 2>&1 &
    sleep 8
    ps -ef | grep "ioMonitor" | grep -v grep > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        for pid in `ps -ef | grep "ioMonitor" | grep -v grep | awk '{print $2}'`
        do
            kill -9 $pid > /dev/null
        done
        echo "Test sysak ioMonitor -T 5 success."
    fi
}

function main() {
    test_help
    test_ioMonitor
    clean
}

main