#!/bin/sh
#
# Usage: sysak iosdiag [options] subcmd [cmdargs]
#        subcmd:
#               latency, io latency diagnosis
#               hangdetect, io hang diagnosis
#        cmdargs:
#               -h, help info
#        options:
#               -u url, transfer datafile to remote url
#               -s latency|[hangdetect], stop diagnosis

temp_log_path="iosdiag_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test $1 failed."
    cat $temp_log_path
    clean
    exit 1
}

function check_metrics() {
    curl "http://127.0.0.1:8400/query/baseQ?gmt=0&selTable=sysom_iolatency&timeLen=5" > $temp_log_path 2>&1 && 
        cat $temp_log_path | grep "sysom_iolatency" > /dev/null
    if (test $? -eq 1)
    then
        handle_failed "iosdiag latency monitor"
    else
        echo "Test sysak iosdiag latency -t 0 -T 5 -m success."
    fi
}

function test_help() {
    sysak iosdiag -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: sysak iosdiag" > /dev/null
    if test $? -ne 0
    then
        handle_failed "iosdiag help"
    else
        echo "Test iosdiag helpinfo success."
    fi
}

function test_iolatency() {
    sysak iosdiag latency -t 0 -T 5 > $temp_log_path 2>&1 && cat $temp_log_path | grep -E "distribution|everything" > /dev/null
    if test $? -ne 0
    then
        handle_failed "iosdiag latency"
    else
        echo "Test sysak iosdiag latency -t 0 -T 5 success."
    fi
}

function test_iolatency_monitor() {
    sysak iosdiag latency -t 0 -T 10 -m
    check_metrics
}

function test_iohang() {
    sysak iosdiag hangdetect -t 1 -T 5 > $temp_log_path 2>&1 && cat $temp_log_path | grep -E "distribution|everything" > /dev/null
    if test $? -ne 0
    then
        handle_failed "iosdiag hangdetect"
    else
        echo "Test sysak iosdiag hangdetect -t 1 -T 5 success."
    fi
}

function test_iodiagnosis() {
    sysak iosdiag iodiagnosis -t 20 -y /etc/sysak/base.yaml -s 'await=1,diagIolat=on,diagIohang=on' > $temp_log_path 2>&1 && 
        cat $temp_log_path | grep -E "diskname|type|report" > /dev/null
    if test $? -ne 0
    then
        handle_failed "iosdiag iodiagnosis"
    else
        echo "Test iosdiag iodiagnosis -t 20 success."
    fi
}

function main() {
    test_help
    test_iolatency
    test_iolatency_monitor
    test_iohang
    # test_iodiagnosis
    clean
}

main