#!/bin/sh
#
# usage: iowaitstat [-h] [-p PID] [-T TIMEOUT] [-t TOP] [-c CYCLE] [-j JSON]
#                   [-w IOWAIT_THRESH]

# Report iowait for tasks.

# optional arguments:
#   -h, --help            show this help message and exit
#   -p PID, --pid PID     Specify the process id.
#   -T TIMEOUT, --Timeout TIMEOUT
#                         Specify the timeout for program exit(secs).
#   -t TOP, --top TOP     Report the TopN with the largest iowait.
#   -c CYCLE, --cycle CYCLE
#                         Specify refresh cycle(secs).
#   -j JSON, --json JSON  Specify the json-format output.
#   -w IOWAIT_THRESH, --iowait_thresh IOWAIT_THRESH
#                         Specify the iowait-thresh to report.

# e.g.
#   ./iowaitstat.py
# 			Report iowait for tasks
#   ./iowaitstat.py -c 1
# 			Report iowait for tasks per secs
#   ./iowaitstat.py -p [PID] -c 1
# 			Report iowait for task with [PID] per 1secs

temp_log_path="iowaitstat_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test iowaitstat failed." 
    cat $temp_log_path
    clean
    exit 1
}

function test_help() {
    sysak iowaitstat -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "usage: iowaitstat" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test iowaitstat helpinfo success."
    fi
}

function test_iowaitstat() {
    sysak iowaitstat > $temp_log_path 2>&1 && cat $temp_log_path | grep "global iowait" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak iowaitstat success."
    fi
}


function main() {
    test_help
    test_iowaitstat
    clean
}

main