#!/bin/sh
#
# usage: wbtrace.py [-h] [-p PID] [-T TIMEOUT] [-t TOP] [-c CYCLE]
#                      [-w IOWAIT_THRESH] [-d DEVICE]

# Report iowait for tasks.

# optional arguments:
#   -h, --help            show this help message and exit
#   -p PID, --pid PID     Specify the process id.
#   -T TIMEOUT, --Timeout TIMEOUT
#                         Specify the timeout for program exit(secs).
#   -t TOP, --top TOP     Report the TopN with the largest iowait.
#   -c CYCLE, --cycle CYCLE
#                         Specify refresh cycle(secs).
#   -d DEVICE --device DEVICE
#                         Specify the disk name.

# e.g.
#   ./wbtrace.py
#       Track the writeback
#   ./wbtrace.py -c 1
#   		Report per secs
#   ./wbtrace.py -p [PID]
#   		Specify the writeback process[with [PID]] 

temp_log_path="wbtrace_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test wbtrace failed." 
    cat $temp_log_path
    clean
    exit 1
}

function test_help() {
    sysak wbtrace -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "usage: wbtrace" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test wbtrace helpinfo success."
    fi
}

function test_wbtrace() {
    sysak wbtrace > $temp_log_path 2>&1 && cat $temp_log_path | grep "inode" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak wbtrace success."
    fi
}


function main() {
    test_help
    test_wbtrace
    clean
}

main