#!/bin/sh

#usage: fcachetop [-h] [-f FILE] [-i INTERVAL] [-T TOP] [-v]
#Statistics the file page cached.
#optional arguments:
#  -h, --help            show this help message and exit
#  -f FILE, --file FILE  Statistics the file pages cached from specified file.
#  -i INTERVAL, --interval INTERVAL
#                        Display the file pages cached per N seconds(CTRL+C
#                        exit).
#  -T TOP, --top TOP     Display the file pages cached of TopN (default Top
#                        10).
#  -v, --verbose         Display the full path of the file(By default, when the
#                        file path exceeds 48 characters, the full path of the
#                        file is hidden).
#e.g.
#./fcachetop             Display the file pages cached of Top10
#./fcachetop -f /xxx/file
#                        Statistics the file pages cached for '/xxx/file'
#./fcachetop -i 2        Display the file pages cached per N seconds(CTRL+C exit)
#./fcachetop -T 30       Display the file pages cached of Top30

temp_log_path="fcachetop_tmp.log"
test_file="fcachetop_test.txt"

function clean() {
    rm $temp_log_path
    rm $test_file
}

function handle_failed() {
    echo "Test fcachetop failed." 
    clean
    exit 1
}

function test_help() {
    sysak fcachetop -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "usage: fcachetop" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test fcachetop helpinfo success."
    fi
}

function test_fcachetop_no_args() {
    sysak fcachetop  > $temp_log_path 2>&1 && cat $temp_log_path | grep "Cached pages/size" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak fcachetop success."
    fi
}

function test_fcachetop_T_15() {
    sysak fcachetop -T 15 > $temp_log_path 2>&1 && cat $temp_log_path | grep "Cached pages/size" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak fcachetop -T 15 success."
    fi
}

function test_fcachetop_f() {
    sysak fcachetop -f $test_file > $temp_log_path 2>&1 && cat $temp_log_path | grep "Cached pages/size" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak fcachetop -f success."
    fi
}


function main() {
    test_help
    test_fcachetop_no_args
    test_fcachetop_T_15
    touch $test_file
    test_fcachetop_f
    clean
}

main
