#!/bin/sh

default_log_path="/var/log/sysak/imc_latency/imc_latency.log"
temp_log_path="temp.log"
redirect_log_path="redirect.log"

function reset() {
    rm -rf $default_log_path
    rm -rf $redirect_log_path
    rm -rf $temp_log_path
}

function test_env() {
    reset
    sysak -g imc_latency > $temp_log_path 2>&1 &
    sleep 5
    pkill imc_latency
    # exit 1
    times=`cat $temp_log_path | grep "Unsupport model" | wc -l`
    if test $times -ne 0
    then
        echo "Test env failed."
        exit 1
    else
        reset
        echo "Test env success."
    fi
}

function test_help() {
    reset
    sysak -g imc_latency --help > $temp_log_path 2>&1&& cat $temp_log_path | grep "USAGE: imc_latency" > /dev/null
    
    if test $? -ne 0
    then
        echo "Test help failed."
        exit 1
    else
        reset
        echo "Test help success."
    fi
}

function test_delay() {
    reset
    sysak -g imc_latency --delay=1 > /dev/null 2>&1 &
    sleep 3
    pkill imc_latency
    times=`cat $default_log_path | grep TIME-STAMP | wc -l`
    
    if test $times -ne 2
    then
        echo "Test delay failed."
        exit 1
    else
        echo "Test delay success."
    fi
}

function test_iter() {
    reset
    sysak -g imc_latency --delay=1 --iter=3 > /dev/null 2>&1 
    times=`cat $default_log_path |grep TIME-STAMP |wc -l`
    if test $times -ne 3 
    then
        echo "Test iter failed"
        exit 1
    else
        echo "Test iter success"
    fi
}

function test_logfile() {
    reset
    sysak -g imc_latency --delay=1 --iter=2 --logfile=$redirect_log_path > /dev/null 2>&1 
    times=`cat $redirect_log_path|grep TIME-STAMP |wc -l`

    if test $times -ne 2
    then
        echo "Test logfile failed"
        exit 1
    else
        echo "Test logfile success"
    fi
}

function main() {
    test_env
    test_help
    test_delay
    test_iter
    test_logfile
    reset
}

main