#!/bin/sh
#   Usage: memcgoffline [OPTIONS]
#   Options:
#   -n PRINT_MAX_CG_NUM   Max offline memcg paths to printf(default 10000)
#   -h HELP               help

#   EXAMPLE:
#   memcgoffline        # display number of offline memcg and all their paths.
#   memcgoffline -n 10  # display number of offline memcg and 10 of offline memcg paths.

temp_log_path="memcgoffline.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test memcgoffline failed." 
    clean
    exit 1
}

function test_help() {
    sysak memcgoffline --help > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: memcgoffline" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test memcgoffline --help success."
    fi
}

function check_fail() {
    check_parse_arg=`cat $temp_log_path | grep "parse arg error" | wc -l`
    check_kcore=`cat $temp_log_path | grep "kcore init failed" | wc -l`
    check_btf=`cat $temp_log_path | grep "btf init failed" | wc -l`
    check_memcg_iter_init=`cat $temp_log_path | grep "memcg_iter_init failed" | wc -l`
    check_offline_caculate=`cat $temp_log_path | grep "caculate offline memcg failed" | wc -l`
    if (test $check_parse_arg -ne 0) || (test $check_kcore -ne 0) || (test $check_btf -ne 0)    \
        || (test $check_memcg_iter_init -ne 0) || (test $check_offline_caculate -ne 0)
    then
        handle_failed  
    fi    
}

function check_content() {
    check_offline_cg_num=`cat $temp_log_path | grep "Offline memory cgroup num" | wc -l`
    check_total_cg_num=`cat $temp_log_path | grep "Total memory cgroup num" | wc -l`
    if (test $check_offline_cg_num -lt 1) || (test $check_total_cg_num -lt 1)
    then
        handle_failed
    fi
}

function test_all() {
    sysak memcgoffline > $temp_log_path 2>&1

    check_fail
    check_content
    echo "Test memcgoffline all success."
}

function test_n() {
    sysak memcgoffline -n 10 > $temp_log_path 2>&1

    check_fail
    check_content
    check_output_num=`cat $temp_log_path | grep "cgroup path" | wc -l`
    if (test $check_output_num -gt 10)
    then
        handle_failed
    fi
    echo "Test memcgoffline -n success."
}


function main() {
    test_help
    test_all
    test_n
    clean
}

main