#!/bin/sh
# help for usage:
#-g: only show memory usage graph
#-f: show file cache usage detail
#-a: show anon page usage detail
#-k: check kernel memleak
#-l: show memory usage of thread
#-c: show memory usage of cgroup
#-j: output for json file
temp_log_path="memgraph.log"
json_path="/var/log/sysak/memgraph/memgraph.json"

function clean() {
    rm $temp_log_path
    if [ -e "$json_path" ]; then
        rm $json_path
    fi
}

function handle_failed() {
    func=$1
    echo "Test memgraph failed." $func
    clean
    exit 1
}

function test_help() {
    sysak memgraph -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "show memory usage graph" > /dev/null
    if test $? -ne 0
    then
        handle_failed "test_help"
    else
        echo "Test memgraph --help success."
    fi
}

function check_fail() {
    check_parse_arg=`cat $temp_log_path | grep "get opt error" | wc -l`
    check_must_root=`cat $temp_log_path | grep "must be run as root" | wc -l`
    if (test $check_parse_arg -ne 0) || (test $check_must_root -ne 0) 
    then
        handle_failed  "check_fail"
    fi    
}

function check_content() {
    check_result=`cat $temp_log_path | grep "userUsed" | wc -l`
    if (test $check_result -eq 0)
    then
        handle_failed "check_content"
    fi
}

function test_noarg() {
    sysak memgraph > $temp_log_path 2>&1
    check_fail "test_noarg"
    check_content
    echo "Test memgraph success."
}

function test_f() {
    sysak memgraph -f > $temp_log_path 2>&1
    lines=`cat $temp_log_path | wc -l`
    check_fail "test_f"
    if (test  $lines -ne 11)
    then
        handle_failed "test_f"
    fi
    echo "Test memgraph -f success."
}

function test_j() {
    sysak memgraph -j $json_path > $temp_log_path 2>&1
    check_fail "test_j"
    check_json=`cat $json_path | grep "memGraph" | wc -l`
    if (test $check_json -lt 1)
    then 
       handle_failed "test_j" 
    fi
    echo "Test memgraph -j success."
}

function test_l() {
    sysak memgraph -l > $temp_log_path 2>&1
    lines=`cat $temp_log_path | wc -l`
    check_fail "test_l"
    if (test $lines -ne 10)
    then
        handle_failed "test_l"
    fi
    echo "Test memgraph -l success."
}

function test_a() {
    sysak memgraph -a > $temp_log_path 2>&1
    lines=`cat $temp_log_path | wc -l`
    check_fail "test_a"
    if (test $lines -ne 10)
    then
        handle_failed "test_a"
    fi
    echo "Test memgraph -a success."
}

function test_k() {
    sysak memgraph -k > $temp_log_path 2>&1
    check_fail "test_k"
    check_json=`cat $temp_log_path | grep "leak" | wc -l`
    if (test $check_json -lt 1)
    then 
       handle_failed "test_k" 
    fi
    echo "Test memgraph -k success."
}

function test_g() {
    sysak memgraph -g > $temp_log_path 2>&1
    check_fail "test_g"
    check_content
    echo "Test memgraph -g success."
}

function test_c() {
    sysak memgraph -c 1 > $temp_log_path 2>&1
    check_fail "test_c"
    check_json=`cat $temp_log_path | grep "/sys/fs/cgroup/memory/user" | wc -l`
    if (test $check_json -lt 1)
    then 
       handle_failed "test_c" 
    fi
    echo "Test memgraph -c success."
}

function main() {
    test_help
    test_noarg
    test_f
    test_g
    test_a
    test_k
    test_l
    test_c
    test_j
    clean
}

main
