#!/bin/sh

#mmaptrace: user mode memory leak monitor tool
#Usage: mmaptrace <option> [<args>]
#  -h                 help information
#  -c                 check main system services status
#  -C                 quickly check user memory leak and display top 5
#  -k                 task skip list with using -C
#  -t                 check time for each task
#  -n                 show task numbers, default is 5
#  -p <pid>           enable memory leak trace
#  -l                 set mmaloc/mmap size
#  -s                 show user mode trace
#  -d                 disable hang check
#example:
#sysak mmaptrace -C -k name1,name2 -t 2 -n 6
#sysak mmaptrace -p 111 -l 1024

temp_log_path="mmaptrace_tmp.log"
calltrace_log="/var/log/sysak/mmaptrace/mmaptrace_log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test sysak mmaptrace failed." 
    clean
    exit 1
}

function test_help() {
    sysak -g mmaptrace -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: mmaptrace <option>" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak mmaptrace helpinfo success."
    fi
}

function test_mmaptrace_c() {
    sysak mmaptrace -c > $temp_log_path 2>&1 && cat $temp_log_path | grep -E "Reclaim|Compaction|Cgroup Reclaim" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak mmaptrace -c success."
    fi
}

function test_mmaptrace_C() {
    sysak mmaptrace -C > $temp_log_path 2>&1 && cat $temp_log_path | grep "RSS size Top 5" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak mmaptrace -C success."
    fi
}

function test_mmaptrace_C_t() {
    sysak mmaptrace -C -t 2 > $temp_log_path 2>&1 && cat $temp_log_path | grep "RSS size Top 5" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak mmaptrace -C -t 2 success."
    fi
}

function test_mmaptrace_C_t_n() {
    sysak mmaptrace -C -t 2 -n 10 > $temp_log_path 2>&1 && cat $temp_log_path | grep "RSS size Top 10" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak mmaptrace -C -t 2 -n 10 success."
    fi
}

function test_mmaptrace_C_t_n_k() {
    sysak mmaptrace -C -t 2 -n 10 -k 1 > $temp_log_path 2>&1 && cat $temp_log_path | grep "RSS size Top 10" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak mmaptrace -C -t 2 -n 10 -k 1 success."
    fi
}

function test_mmaptrace_calltrace() {
    polkitd_pid=`ps -aux | grep polkitd | grep -v "grep"| awk -F " " '{print $2}'`
    sysak mmaptrace -p $polkitd_pid -l 1024 
    sysak mmaptrace -s 
    cat $calltrace_log | grep -E "name|pid" > /dev/null
    if test $? -ne 0
    then
        sysak mmaptrace -d
        handle_failed
    else
        sysak mmaptrace -d
        echo "Test sysak mmaptrace -p xx -l 1024 success."
    fi
}

function main() {
    test_help
    test_mmaptrace_C
    test_mmaptrace_C_t
    test_mmaptrace_C_t_n
    test_mmaptrace_C_t_n_k
    test_mmaptrace_calltrace
    clean
}

main
