#!/bin/sh
# help for usage:
#-h --help     print the help
#-f --dmesg file
#-l --live mode
#-t --time mode
#-j --output json
#-n --# of output results

temp_log_path="oomcheck.log"
tmp_oomlog_path="test_case/feature/sysak/mem-test/oomexample.txt"
json_path="/var/log/sysak/oomcheck/oomcheck.json"

function clean() {
    rm $temp_log_path
    if [ -e "$json_path" ]; then
        rm $json_path
    fi
}

function handle_failed() {
    func=$1
    echo "Test oomcheck failed." $1
    clean
    exit 1
}

function test_help() {
    sysak oomcheck -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "for example:" > /dev/null
    if test $? -ne 0
    then
        handle_failed "test_help"
    else
        echo "Test oomcheck --help success."
    fi
}

function check_fail() {
    func=$1
    check_memgraph=`cat $temp_log_path | grep "get_memgraph err" | wc -l`
    check_dmesg_analyze=`cat $temp_log_path | grep "oom_dmesg_analyze failed" | wc -l`
    check_oom_diagnose=`cat $temp_log_path | grep "oom_diagnose failed" | wc -l`
    check_reason_analyze=`cat $temp_log_path | grep "oom_reason_analyze err" | wc -l`
    if (test $check_memgraph -ne 0) || (test $check_dmesg_analyze -ne 0) || (test $check_oom_diagnose -ne 0)    \
        || (test $check_reason_analyze -ne 0)
    then
        handle_failed  $1
    fi    
}

function check_content() {
    json=$1
    local_log_file="$temp_log_path"
    if [ "$json" = "true" ]; then
        check_json=`cat $local_log_file | grep "rss_list_desc" | wc -l`
        if (test $check_json -eq 0)
        then
            handle_failed "check_content"
        fi
        return 0
    fi

    check_meminfo=`cat $local_log_file | grep "mem info" | wc -l`
    if (test $check_meminfo -eq 0)
    then
        handle_failed "check_content2"
    fi

    check_result=`cat $local_log_file | grep "diagnones result:" | wc -l`
    if (test $check_result -eq 0)
    then
        handle_failed "check_content3"
    fi

}

function test_noarg() {
    sysak oomcheck > $temp_log_path 2>&1
    check_fail "test_noarg"
    echo "Test oomcheck success."
}

function test_f() {
    sysak oomcheck -f $tmp_oomlog_path > $temp_log_path 2>&1
    check_fail "test_f"
    check_content "false"
    echo "Test oomcheck -f success."
}

function test_j() {
    sysak oomcheck -f $tmp_oomlog_path -j > $temp_log_path 2>&1
    check_fail "test_j"
    check_content "true"
    echo "Test oomcheck -j success."
}

function test_l() {
    sysak oomcheck -l > $temp_log_path 2>&1
    check_fail "test_l"
    echo "Test oomcheck -l success."
}

function test_t() {
    sysak oomcheck -f $tmp_oomlog_path -t 1986880 > $temp_log_path 2>&1
    check_fail "test_t"
    check_content "false"
    echo "Test oomcheck -t success."
}

function test_n() {
    sysak oomcheck -f $tmp_oomlog_path -n 10 > $temp_log_path 2>&1
    check_fail "test_n"
    lines=`cat $temp_log_path | grep "total oom" | wc -l`
    if (test $lines -le 1)
    then
        handle_failed "test_n"
    fi
    echo "Test oomcheck -n 10 success."
}

function main() {
    test_help
    test_noarg
    test_f
    test_l
    test_t
    test_j
    test_n
    clean
}

main
