#!/bin/sh
# help for usage:
# -p: analysis pod pagecache(sysak podmem -p ack-node-problem-detector)
# -c: analysis container pagecache(sysak podmem -c bd2146176a5ce)
# -a: analysis all container pagecache(sysak podmem -f /sys/fs/cgroup/memory/system.slice)
# -f: analysis cgroup pagecache(sysak podmem -f /sys/fs/cgroup/memory/system.slice)
# -s: analysis system pagecache(sysak podmem -s )
# -j: dump result to json file (sysak podmem -s -j ./test.json)
# -r: set sample rate ,default set to 1 (sysak podmem -s -r 2)
# -t: output filecache top ,default for top 10 (sysak podmem -s -t 20)
# -m: analysis pod recv-Q  and send-Q memory

temp_log_path="podmem.log"
json_path="/var/log/sysak/podmem/podmem.json"

function clean() {
    rm $temp_log_path
    if [ -e "$json_path" ]; then
        rm $json_path
    fi
}

function handle_failed() {
    echo "Test podmem failed." 
    clean
    exit 1
}

function test_help() {
    sysak podmem -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "help for usage:" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test podmem --help success."
    fi
}

function check_fail() {
    check_parse_arg=`cat $temp_log_path | grep "get opt error" | wc -l`
    check_must_root=`cat $temp_log_path | grep "must be root" | wc -l`
    check_pageshift=`cat $temp_log_path | grep "failed to page shift" | wc -l`
    check_setup=`cat $temp_log_path | grep "failed to setup" | wc -l`
    if (test $check_parse_arg -ne 0) || (test $check_must_root -ne 0) || (test $check_pageshift -ne 0)    \
        || (test $check_setup -ne 0)
    then
        handle_failed  
    fi    
}

function check_content() {
    top=$1
    cgroup=$2
    local_log_file="$temp_log_path"
    if [ ! -z "$3" ]; then
        local_log_file="$3"
    fi

    check_cached=`cat $local_log_file | grep "cached" | wc -l`
    if (test $check_cached -gt $top) || (test $check_cached -eq 0)
    then
        handle_failed
    fi

    check_cgcached=`cat $local_log_file | grep "cgcached" | wc -l`
    if [ "$cgroup" = "true" ]; then
        if (test $check_cgcached -gt $top) || (test $check_cgcached -eq 0)
        then
            handle_failed
        fi
    fi
}

function test_all() {
    sysak podmem > $temp_log_path 2>&1
    sleep 3
    check_fail
    if command -v docker &> /dev/null \
        || command -v crictl &> /dev/null
    then
        check_content 10 "true"
    else
        echo "Container runtime not exist, ignore content check."
    fi
    echo "Test podmem -a success."
}

function test_s() {
    sysak podmem -s > $temp_log_path 2>&1
    sleep 4
    check_fail
    check_content 10 "false"
    echo "Test podmem -s success."
}

function test_f() {
    path="/sys/fs/cgroup/memory"
    if [ -e "$path" ]; then
        sysak podmem -f /sys/fs/cgroup/memory > $temp_log_path 2>&1
        sleep 3
        check_fail
        check_content 10 "true"
    else
        echo "Memory cgroup not enable, ignore podmem -f check."
    fi
    echo "Test podmem -f /sys/fs/cgroup/memory success."
}

function test_j() {
    sysak podmem -s -t 5 -j $json_path > $temp_log_path 2>&1
    sleep 2
    check_fail
    if [ -f "$json_path" ]; then
        check_content 10 "false" "$json_path"
    else
        handle_failed
    fi
    echo "Test podmem -s -t 5 -j success."
}

function test_r() {
    sysak podmem -s -r 100 > $temp_log_path 2>&1
    sleep 2
    check_fail
    check_content 10 "false"
    echo "Test podmem -s -r 100 success."
}

function test_t() {
    sysak podmem -s -t 5 > $temp_log_path 2>&1
    sleep 5
    check_fail
    check_content 5 "false"
    echo "Test podmem -s -t 5 success."
}

function test_m() {
    sysak podmem -m > $temp_log_path 2>&1
    lines=`cat $temp_log_path | wc -l`
    if test $lines -ne 2
    then
        handle_failed
    fi
}

function main() {
    test_help
    test_all
    test_s
    test_f
    test_j
    test_r
    test_t
    test_m
    clean
}

main