#!/bin/sh


# PingTrace
# Usage: /usr/local/sysak/.sysak_components/tools/pingtrace [OPTIONS]

# Options:
#   -v,--version                Display program version information and exit
#   -h,--help                   print this help message and exit
#   -s,--server                 run server
#   -c,--client ip              run client
#   -C,--count UINT             packet send count, default is infinity
#   -i interval_us              set send interval_us per ip in us, default is 1s
#   -t UINT                     set pingtrace run time in second, default is infinity
#   -A                          ping adaptive, would be flooding
#   -m,--maxdelay us            max_delay in us, if ping's delay is larger than max_delay, print it. default is 0
#   -b INT=556                  packet size, at least 144
#   --log TEXT=./pingtrace.log  log name
#   --logsize INT               log max size in bytes, default is 2M
#   --logbackup INT=3           log max backup num
#   --mode auto/pingpong/compact
#                               pingtrace mode, optional:compact, pingpong, or auto
#   -o,--output image/json/log/imagelog
#                               output mode, optional:image, json, log, imagelog
#   -n,--namespace              get net namespace infos
#   --nslocal                   to tell PingTrace that local and remote is on same host
#   --userid UINT               specify user id to distinguish different node
#   --btf_path TEXT             specify custom btf vmlinux path
#   --compat                    compat with kernel 4.9
#   --debug                     used for debug, output more detail messages
temp_log_path="pingtrace.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test pingtrace failed." 
    clean
    exit 1
}

function test_help() {
    sysak -g pingtrace --help > $temp_log_path 2>&1 && cat $temp_log_path | grep "PingTrace" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test pingtrace --help success."
    fi
}

function test_trace() {
    sysak -g pingtrace -c 127.0.0.1 -C 1 > $temp_log_path && cat $temp_log_path | grep "meta" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test pingtrace success."
    fi
}


function main() {
    test_help
    test_trace
    clean
}

main
