#!/bin/sh


# rtrace 0.2.0
# Diagnosing tools of kernel network

# USAGE:
#     rtrace [FLAGS] [OPTIONS] <SUBCOMMAND>

# FLAGS:
#     -h, --help       Prints help information
#     -V, --version    Prints version information
#     -v, --verbose    Verbose debug output

# OPTIONS:
#         --btf <btf>    Custom btf path

# SUBCOMMANDS:
#     delta      Store or display delta information
#     drop       Packet drop diagnosing
#     help       Prints this message or the help of the given subcommand(s)
#     latency    Packet latency tracing
#     retran     Tracing retransmit

temp_log_path="rtrace.log"

function clean() {
    rm $temp_log_path
    pkill ping
}

function handle_failed() {
    echo "Test rtrace failed." 
    clean
    exit 1
}

function test_help() {
    sysak -g rtrace --help > $temp_log_path 2>&1 && cat $temp_log_path | grep "rtrace" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test rtrace --help success."
    fi
}

function test_drop() {
    sysak -g rtrace drop --duration 3 > $temp_log_path && cat $temp_log_path | grep "netstat" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test rtrace drop success."
    fi
}

function test_latency() {
    sysak -g rtrace latency --proto icmp --duration 3 > $temp_log_path && cat $temp_log_path | grep "SinceBootTimeDuration" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test rtrace latency success."
    fi
}


function test_retran() {
    sysak -g rtrace retran --duration 3 > $temp_log_path && cat $temp_log_path | grep "netstat" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test rtrace retran success."
    fi
}


function main() {
    ping 127.0.0.1 &
    test_help
    test_drop
    test_latency
    test_retran
    clean
}

main
