#!/bin/sh


# sysak btf: Extract the btf file to the specified directory
# options: -h, help information
#          -d, Specify the path, default directory path: /usr/local/sysak/.sysak_components/tools/4.19.91-007.ali4000.alios7.x86_64
#          -l, show vmlinux btf list
#          -g, Specify the vmlinux file path, which is used to generate btf. The generated btf file path is: /usr/local/sysak/.sysak_components/tools/vmlinux-4.19.91-007.ali4000.alios7.x86_64
temp_log_path="btf.log"
uname=`uname -r`

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test btf failed." 
    clean
    exit 1
}

function test_help() {
    sysak btf -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "btf" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test btf --help success."
    fi
}

function test_gen() {
    wget https://mirrors.aliyun.com/alinux/2.1903/plus/x86_64/debug/kernel-debuginfo-common-x86_64-4.19.91-27.1.al7.x86_64.rpm?spm=a2c6h.25603864.0.0.4a6b451dWsSK3n
    wget https://mirrors.aliyun.com/alinux/2.1903/plus/x86_64/debug/kernel-debuginfo-4.19.91-27.1.al7.x86_64.rpm?spm=a2c6h.25603864.0.0.3fb7451dhlQK7Y
    rpm -ivh kernel-debuginfo-common-x86_64-4.19.91-27.1.al7.x86_64.rpm?spm=a2c6h.25603864.0.0.4a6b451dWsSK3n --force
    rpm -ivh kernel-debuginfo-4.19.91-27.1.al7.x86_64.rpm?spm=a2c6h.25603864.0.0.3fb7451dhlQK7Y --force

    sysak btf -d ./ -g /usr/lib/debug/lib/modules/4.19.91-27.1.al7.x86_64/vmlinux

    ls > $temp_log_path 2>&1 && cat $temp_log_path | grep "vmlinux" > /dev/null
    
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test btf gen --help success."
    fi
}

function main() {
    yum install -y kernel-debuginfo

    test_help
    test_gen
    clean
}

main
