#!/bin/sh

temp_log_path="temp.log"
default_log_path="/var/log/sysak/irqoff/irqoff.log"
# Usage: irqoff [OPTION...]
# Catch the irq-off time more than threshold.

# USAGE: irqoff [--help] [-t THRESH(ms)] [-S SHM] [-f LOGFILE] [duration(s)]

# EXAMPLES:
#     irqoff                # run forever, and detect irqoff more than
# 10ms(default)
#     irqoff -S shmkey      # record summary log to share memory shmkey
#     irqoff -t 15          # detect irqoff with threshold 15ms (default 10ms)
#     irqoff -f a.log       # record result to a.log (default to
# ~sysak/irqoff/irqoff.log)

#   -f, --logfile=LOGFILE      logfile for result
#   -S, --summary=SHMKEY       Record summary log to share memory
#   -t, --threshold=THRESH     Threshold to detect, default 10ms
#   -v, --verbose              Verbose debug output
#   -?, --help                 Give this help list
#       --usage                Give a short usage message
#   -V, --version              Print program version

# Mandatory or optional arguments to long options are also mandatory or optional
# for any corresponding short options.

handle_error() {
    echo "Test Failed: $1"
    reset
    exit 1
}

reset() {
    pkill irqoff
    rm -rf $temp_log_path
}

# irqoff --help
test_help() {
    reset
    val=$(sysak -g irqoff --help | grep 'USAGE: irqoff' -c)

    if test $val -eq 1; then
        echo "Test help success."
    else
        handle_error "sysak -g irqoff --help"
    fi
}

test_file() {
    reset
    sysak -g irqoff -f $temp_log_path &
    sleep 5
    pkill irqoff --signal SIGINT
    num=$(cat $temp_log_path | grep "TIME(irqoff)" -c)
    if test $num -ne 0; then
        echo "Test file success."
    else
        handle_error "sysak -g irqoff -f"
    fi
}

test_normal() {
    reset
    sysak -g irqoff &
    sleep 5
    pkill irqoff --signal SIGINT
    num=$(cat $default_log_path | grep "TIME(irqoff)" -c)
    if test $num -ne 0; then
        echo "Test normal success."
    else
        handle_error "sysak -g irqoff"
    fi
}

test_verbose() {
    reset
    sysak -g irqoff -v -f $temp_log_path >/dev/null 2>&1 &
    sleep 5
    pkill irqoff --signal SIGINT

    num=$(cat $temp_log_path | grep "TIME(irqoff)" -c)
    if test $num -ne 0; then
        echo "Test verbose success."
    else
        handle_error "sysak -g irqoff -v -f $temp_log_path"
    fi
}

test_threshold() {
    reset
    sysak -g irqoff -t 30 -f $temp_log_path &
    sleep 5
    pkill irqoff --signal SIGINT
    num=$(cat $temp_log_path | grep "TIME(irqoff)" -c)
    if test $num -ne 0; then
        echo "Test threshold success."
    else
        handle_error "sysak -g irqoff -t 30 -f $temp_log_path"
    fi
}

test_S() {
    reset
    # if [ -f sysak_shmem_create ]; then
    if [ -f ../../../../dep/schedule/sysak_shmem_create ]; then
        ../../../../dep/schedule/sysak_shmem_create &
    else
        echo "sysak_shmem_create not exist, didnt end"
        handle_error "-S"
    fi
    pid_shm=$(pidof sysak_shmem_create)
    if [ $pid_shm ]; then
        echo "end"
    else
        echo $pid_shm
        echo "didnt end"
        handle_error "-S"
    fi
    sysak -g irqoff -S noschedselftest >$temp_log_path 2>&1 &

    pkill irqoff --signal SIGINT
    pkill sysak_shmem_create
    num=$(cat $temp_log_path | grep "shm_open " -c)
    if test $num -eq 0;then
        echo "Test -S success."
    else
        handle_error "-S"
    fi
}

main() {
    test_help
    test_normal
    test_file
    test_verbose
    test_threshold
    test_S
    reset
}

main
