#!/bin/sh

#sysak loadtask: show all tasks of load contribution
#options: -h,          help information
#         -m maxload,  only show tasks when load reach maxload 
#         -f datafile, file for output
#         -i interval, the interval checking the load
#         -d,          keep monitoring even if greater than maxload occurs.useful only if the -m option is set
#         -s,          show summary result. will insmod loadtask.ko
#         -k,          terminate running loadtask which started previously
#         -g,          default collect cpu perf flamegraph by cpu_flamegraph tool
#         -r datafile, read datafile created by '-f datafile' or by default(datafile directory /var/log/sysak/loadtask/) and show result

temp_log_path="loadtask_tmp.log"
loadtask_tmp_file="/var/log/sysak/loadtask/.tmplog"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test loadtask failed." 
    clean
    exit 1
}

function test_help() {
    sysak loadtask -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "sysak loadtask: show" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test loadtask helpinfo success."
    fi
}

function test_sysom_diagnosis() {
    sysak loadtask -s -g > $temp_log_path 2>&1 && cat $temp_log_path | grep -E "top load tasks|svg" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test loadtask -s -g success."
    fi
}

function test_loadtrask_r() {
    sysak loadtask -r $loadtask_tmp_file > $temp_log_path 2>&1 && cat $temp_log_path | grep -E "top load tasks" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test loadtask -r success."
    fi
}

function test_loadtask_f() {
    sysak loadtask -s -f $temp_log_path && cat $temp_log_path | grep -E "running tasks" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test loadtask -s -f success."
    fi
}

function test_loadtask_monitor() {
    sysak loadtask -m 10 -i 2 -d 100 &
    ps | grep loadtask
    if test $? -ne 0
    then
        handle_failed
    fi

    sysak loadtask -k
    ps | grep loadtask
    if test $? -eq 0
    then
        handle_failed
    else
        echo "Test lsysak loadtask -m 10 -i 2 -d 100 and sysak loadtask -k success."
    fi
}

function main() {
    test_help
    test_sysom_diagnosis
    test_loadtrask_r
    test_loadtask_f
    #test_loadtask_monitor
    clean
}

main
