#!/bin/sh
#  Usage: sysak nosched [OPTIONS]
#  Options:
#  -t THRESH_TIME       specify the threshold time(ms), default=10ms
#  -f result.log        result file, default is /var/log/sysak/nosched.log
#  -s TIME              specify how long to run 
#  -S shmkey            record summary log to share memory

temp_log_path="nosched.log"

function clean() {
    pkill nosched
    rm $temp_log_path
}

function handle_failed() {
    echo "Test nosched failed." 
    clean
    exit 1
}

function check_fail() {
    check_start=`cat $temp_log_path | grep "Failed to open and load BPF skeleton" | wc -l`
    check_syms_cache=`cat $temp_log_path | grep "Failed to load and verify BPF skeleton" | wc -l`
    check_bpf=`cat $temp_log_path | grep "failed to load kallsyms" | wc -l`
    a1=`cat $temp_log_path | grep "\\}" | wc -l`
    a2=`cat $temp_log_path | grep "\\{" | wc -l`
    if (test $check_start -ne 0) || (test $check_syms_cache -ne 0) || (test $check_bpf -ne 0) || [ "$a1" != "$a2" ]
    then
        handle_failed  
    fi    
}

function test_help() {
    sysak -g nosched --help > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: sysak nosched" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test nosched --help success."
    fi
}

function test_all() {
    sysak -g nosched > $temp_log_path 2>&1 &
    sleep 1
    pkill nosched
    sleep 1
    
    check_fail
    echo "Test nosched success."
}

function test_t() {
    sysak -g nosched -t 50 > $temp_log_path 2>&1 &
    sleep 1
    pkill nosched
    sleep 1
    
    check_fail
    echo "Test nosched -t success."
}

function test_f() {
    tmpf=$(mktemp)
    sysak -g nosched -f $tmpf > $temp_log_path 2>&1 &
    sleep 1
    pkill nosched
    sleep 1

    if [ -f $tmpf ]; then
        echo "end"
	rm $tmpf
    else
	rm $tmpf
	handle_failed
    fi

    check_fail
    echo "Test nosched -f success."
}

function test_s() {
    sysak -g nosched -s 4 > $temp_log_path 2>&1 &
    sleep 5

    pid1=`pidof nosched`
    if [ ! $pid1 ]; then
        echo "end"  
    else  
        echo $pid1
        echo "didnt end"  
        handle_failed
    fi 

    check_fail
    echo "Test nosched -s success."
}

function test_S() {
    if [ -f ../../../../dep/schedule/sysak_shmem_create ]; then
        ../../../../dep/schedule/sysak_shmem_create &
    else
        echo "sysak_shmem_create not exist, didnt end"  
	handle_failed
    fi
    pid_shm=`pidof sysak_shmem_create`
    if [ $pid_shm ]; then
        echo "end"  
    else  
        echo $pid_shm
        echo "didnt end"  
        handle_failed
    fi 
    sysak nosched -S noschedselftest  > $temp_log_path 2>&1 &

    pkill nosched
    pkill sysak_shmem_create

    check_fail
    echo "Test nosched -S success."
}

function main() {
    test_help
    test_all
    test_t
    test_f
    test_s
    test_S
    clean
}

main
