#!/bin/sh

# USAGE: rtdelay [--help] [-p PID]  [-d DURATION] [-s ServerPID]
# EXAMPLES:
#     rtdelay                     # trace RT time until Ctrl-C
#     rtdelay -p 185              # only trace threads for PID 185
#     rtdelay -p 185 -s 827732    # only trace threads for PID 185 and server PID
# is 827732
#     rtdelay -d 10               # trace for 10 seconds only

#   -d, --duration=DURATION    Total duration of trace in seconds
#   -p, --pid=PID              Trace this PID only
#   -s, --server=SeverPID      Trace Server PID
#   -?, --help                 Give this help list
#       --usage                Give a short usage message
#   -V, --version              Print program version
temp_log_path="rtdelay.log"

function clean() {
    pkill rtdelay
    rm $temp_log_path
}

function handle_failed() {
    echo "Test rtdelay failed." 
    clean
    exit 1
}

function check_fail() {
    check_start=`cat $temp_log_path | grep "Failed to open and load BPF skeleton" | wc -l`
    check_syms_cache=`cat $temp_log_path | grep "Failed to load and verify BPF skeleton" | wc -l`
    check_bpf=`cat $temp_log_path | grep "failed to load kallsyms" | wc -l`
    a1=`cat $temp_log_path | grep "\\}" | wc -l`
    a2=`cat $temp_log_path | grep "\\{" | wc -l`
    if (test $check_start -ne 0) || (test $check_syms_cache -ne 0) || (test $check_bpf -ne 0) || [ "$a1" != "$a2" ]
    then
        handle_failed  
    fi    
}

function test_help() {
    sysak -g rtdelay --help > $temp_log_path 2>&1 && cat $temp_log_path | grep "USAGE: rtdelay" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test rtdelay --help success."
    fi
}

function test_all() {
    sysak -g rtdelay > $temp_log_path 2>&1 &
    sleep 1
    pkill rtdelay
    sleep 1
    
    check_fail
    echo "Test rtdelay success."
}

function test_p() {
    pid1=`pidof mysqld`
    if [ ! $pid1 ]; then  
        echo "No mysqld diagnose pid=1"  
        pid1=1
    else  
        echo "Diagnose mysqld"  
    fi 
    sysak -g rtdelay -p $pid1 > $temp_log_path 2>&1 &
    sleep 1
    pkill rtdelay
    sleep 1
    
    check_fail
    echo "Test rtdelay -p success."
}

function test_d() {
    sysak -g rtdelay -d 1 > $temp_log_path 2>&1 &
    sleep 5

    pid1=`pidof rtdelay`
    if [ ! $pid1 ]; then  
        echo "end"  
    else  
        echo $pid1
        echo "didnt end"  
        handle_failed
    fi 

    check_fail
    echo "Test rtdelay -d success."
}

function test_s() {
    pid1=`pidof mysqld`
    if [ ! $pid1 ]; then  
        echo "No mysqld, diagnose pid=1"  
        pid1=1
    else  
        echo "Diagnose mysqld"  
    fi 
    sysak -g rtdelay -s $pid1 > $temp_log_path 2>&1 &
    sleep 1
    pkill rtdelay
    sleep 1
    check_fail
    echo "Test rtdelay -s success."
}


function main() {
    test_help
    test_all
    test_p
    test_d
    test_s
    clean
}

main