#!/bin/sh

temp_log_path="temp.log"
default_log_path="/var/log/sysak/runqslow/runqslow.log"
# sysak -g runqslower --help
#Usage: runqslower [OPTION...]
#Trace high run queue latency.
#
#USAGE: runqslower [-h] [-s SPAN] [-t TID] [-S SUM] [-f LOG] [-P] [threshold]
#
#EXAMPLES:
#    runqslower           # trace latency higher than 50ms (default)
#    runqslower -f a.log  # record result to a.log (default
#~/sysak/runqslow/runqslow.log)
#    runqslower 12        # trace latency higher than 12 ms
#    runqslower -p 123    # trace pid 123
#    runqslower -t 123    # trace tid 123 (use for threads only)
#    runqslower -s 10     # monitor for 10 seconds
#    runqslower -S shmkey # Record summary log to share memory shmkey
#    runqslower -P        # also show previous task name and TID

#  -f, --logfile=LOGFILE      logfile for result
#  -p, --pid=PID              Process PID to trace
#  -P, --previous             also show previous task name and TID
#  -s, --span=SPAN            How long to run
#  -S, --summary=SHMKEY       record summary log to share memory
#  -t, --tid=TID              Thread TID to trace
#  -v, --verbose              Verbose debug output
#  -?, --help                 Give this help list
#      --usage                Give a short usage message
#  -V, --version              Print program version

handle_error() {
    echo "Test Failed: $1"
    reset
    exit 1
}

reset() {
    pkill runqslower
    rm -rf $temp_log_path
}

function check_fail() {
    check_start=`cat $temp_log_path | grep "failed to load kallsyms" | wc -l`
    check_syms_cache=`cat $temp_log_path | grep "failed to create syms_cache" | wc -l`
    check_bpf=`cat $temp_log_path | grep "failed to attach BPF programs" | wc -l`
    if (test $check_start -ne 0) || (test $check_syms_cache -ne 0) || (test $check_bpf -ne 0)
    then
	handle_error
    fi
}

# runqslower --help
test_help() {
    #reset
    val=$(sysak -g runqslower --help | grep 'USAGE: runqslower' -c)

    if test $val -eq 1; then
        echo "Test help success."
    else
        handle_error "sysak -g runqslower --help"
    fi
}

test_file() {
    sysak -g runqslower -f $temp_log_path &
    sleep 5
    pkill runqslower --signal SIGINT
    sleep 1
    num=$(cat $temp_log_path | grep "TIME(runslw)" -c)
    if test $num -ne 0; then
        echo "Test file success."
    else
        handle_error "sysak -g runqslower -f"
    fi
}

test_normal() {
    sysak -g runqslower &
    sleep 3
    pkill runqslower --signal SIGINT
    sleep 1
    num=$(cat $default_log_path | grep "TIME(runslw)" -c)
    if test $num -ne 0; then
        echo "Test normal success."
    else
        handle_error "sysak -g runqslower"
    fi
}

test_tid() {
	pid=$(pidof sleep|awk '{print $1}')
	sysak -g runqslower -t $pid -f $default_log_path >/dev/null 2>&1 &
	sleep 3
	pkill runqslower --signal SIGINT
	sleep 1
	num=$(cat $default_log_path | grep "TIME(runslw)" -c)
	if test $num -ne 0; then
		echo "Test tid success."
	else
		handle_error "sysak -g runqslower -t"
	fi
}

test_pid() {
	sysak -g runqslower -p $pid -f $default_log_path >/dev/null 2>&1 &
	sleep 3
	pkill runqslower --signal SIGINT
	sleep 1
	num=$(cat $default_log_path | grep "TIME(runslw)" -c)
	if test $num -ne 0; then
		echo "Test pid success."
	else
		handle_error "sysak -g runqslower -p"
	fi
}

test_verbose() {
    sysak -g runqslower -v -f $temp_log_path >/dev/null 2>&1 &
    sleep 5
    pkill runqslower --signal SIGINT
    sleep 1
    num=$(cat $temp_log_path | grep "TIME(runslw)" -c)
    if test $num -ne 0; then
        echo "Test verbose success."
    else
        handle_error "sysak -g runqslower -v -f $temp_log_path"
    fi
}

test_threshold() {
    sysak -g runqslower 1 &
    sleep 5
    pkill runqslower --signal SIGINT
    sleep 1
    num=$(cat $default_log_path | grep "TIME(runslw)" -c)
    if test $num -ne 0; then
        echo "Test threshold success."
    else
    	handle_error "sysak -g runqslower 1 (thresh)"
    fi
}

function test_time() {
    sysak -g runqslower -s 1 &
    sleep 6

    pid1=`pidof runqslower`
    if [ $pid1 ]; then
        echo "sysak -g runqslower 1 failed: $pid1"
        handle_failed
    fi

    check_fail
    echo "Test time success."
}

main() {
    test_help
    test_normal
    test_threshold
    test_time
    test_file
    test_pid
    test_tid
    test_verbose
    reset
}

main
