#!/bin/sh

temp_log_path="temp.log"
default_json_path="/var/log/sysak/schedmoni/schedmoni.json"
default_log_path_irqof="/var/log/sysak/schedmoni/irqoff.log"
default_log_path_nosh="/var/log/sysak/schedmoni/nosched.log"
default_log_path_rqslw="/var/log/sysak/schedmoni/runslow.log"

# sysak schedmoni --help
#Usage: schedmoni [OPTION...]
#Trace schedule latency.
#
#USAGE: schedmoni [--help] [-s SPAN] [-t TID] [-c COMM] [-P] [-j|-f LOGFILE]
#[threshold]
#
#EXAMPLES:
#  schedmoni          # trace latency higher than 50 ms (default)
#  schedmoni -f a.log # result to a.log (default
#~sysak/schedmoni/schedmoni.log)
#  schedmoni 20       # trace latency higher than 20 ms
#  schedmoni -p 123   # trace pid 123
#  schedmoni -t 123   # trace tid 123 (use for threads only)
#  schedmoni -s 10    # monitor for 10 seconds
#  schedmoni -P       # also show previous task name and TID
#  schedmoni -j       # record result as json,exclusive with -f
#
#  -c, --comm=COMM            Name of the application
#  -f, --logfile=LOGFILE      logfile for result
#  -j, --json                 record the result with JSON mode
#  -p, --pid=PID              Process PID to trace
#  -P, --previous             also show previous task name and TID
#  -s, --span=SPAN            How long to run
#  -t, --tid=TID              Thread TID to trace
#  -v, --verbose              Verbose debug output
#  -?, --help                 Give this help list
#      --usage                Give a short usage message
#  -V, --version              Print program version

handle_error() {
    echo "Test Failed: $1"
    reset
    exit 1
}

reset() {
    pkill schedmoni
    rm -rf $temp_log_path
}

function check_fail() {
    check_start=`cat $temp_log_path | grep "failed to load kallsyms" | wc -l`
    check_syms_cache=`cat $temp_log_path | grep "failed to create syms_cache" | wc -l`
    check_bpf=`cat $temp_log_path | grep "failed to attach BPF programs" | wc -l`
    if (test $check_start -ne 0) || (test $check_syms_cache -ne 0) || (test $check_bpf -ne 0)
    then
	handle_error
    fi
}

# schedmoni --help
test_help() {
    #reset
    val=$(sysak -g schedmoni --help | grep 'USAGE: schedmoni' -c)

    if test $val -eq 1; then
        echo "Test help success."
    else
        handle_error "sysak -g schedmoni --help"
    fi
}

test_json() {
    #irqf nscf rswf
    sysak -g schedmoni -j >/dev/null 2>&1 &
    sleep 5
    pkill schedmoni --signal SIGINT
    sleep 1
    num=$(cat $default_json_path  | grep "datasources" -c)
    if test $num -ne 0; then
        echo "Test json success."
    else
        handle_error "sysak -g schedmoni -j"
    fi
    rm ${temp_log_path}.rswf ${temp_log_path}.irqf ${temp_log_path}.nscf -f
}

test_file() {
    #irqf nscf rswf
    sysak -g schedmoni -f $temp_log_path >/dev/null 2>&1 &
    sleep 5
    pkill schedmoni --signal SIGINT
    sleep 1
    num=$(cat ${temp_log_path}.rswf | grep "TIME(runslw)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni file"
    fi
    num=$(cat ${temp_log_path}.irqf | grep "TIME(irqoff)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni file"
    fi
    num=$(cat ${temp_log_path}.nscf | grep "TIME(nosched)" -c)
    if test $num -ne 0; then
        echo "Test file success."
    else
        handle_error "sysak -g schedmoni -f"
    fi
    rm ${temp_log_path}.rswf ${temp_log_path}.irqf ${temp_log_path}.nscf -f
}

test_normal() {
    sysak -g schedmoni >/dev/null 2>&1 &
    sleep 3
    pkill schedmoni --signal SIGINT
    sleep 1
    #irqof,nosh,rqslw
    #nosched,irqoff,runslw
    num=$(cat $default_log_path_rqslw | grep "TIME(runslw)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni normal"
    fi
    num=$(cat $default_log_path_irqof | grep "TIME(irqoff)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni normal"
    fi
    num=$(cat $default_log_path_nosh | grep "TIME(nosched)" -c)
    if test $num -ne 0; then
        echo "Test normal success."
    else
        handle_error "sysak -g schedmoni normal"
    fi
}

test_tid() {
	pid=$(pidof sleep|awk '{print $1}')
	sysak -g schedmoni -t $pid -f $default_log_path >/dev/null 2>&1 &
	sleep 3
	pkill schedmoni --signal SIGINT
	sleep 1
    num=$(cat $default_log_path_rqslw | grep "TIME(runslw)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni -t "
    fi
    num=$(cat $default_log_path_irqof | grep "TIME(irqoff)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni -t "
    fi
    num=$(cat $default_log_path_nosh | grep "TIME(nosched)" -c)
    if test $num -ne 0; then
        echo "Test tid success."
    else
        handle_error "sysak -g schedmoni -t "
    fi
}

test_pid() {
	sysak -g schedmoni -p $pid -f $default_log_path >/dev/null 2>&1 &
	sleep 3
	pkill schedmoni --signal SIGINT
	sleep 1
    num=$(cat $default_log_path_rqslw | grep "TIME(runslw)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni -p "
    fi
    num=$(cat $default_log_path_irqof | grep "TIME(irqoff)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni -p "
    fi
    num=$(cat $default_log_path_nosh | grep "TIME(nosched)" -c)
    if test $num -ne 0; then
        echo "Test pid success."
    else
        handle_error "sysak -g schedmoni -p "
    fi
}

test_verbose() {
    sysak -g schedmoni -v >/dev/null 2>&1 &
    sleep 5
    pkill schedmoni --signal SIGINT
    sleep 1
    num=$(cat $default_log_path_rqslw | grep "TIME(runslw)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni -v "
    fi
    num=$(cat $default_log_path_irqof | grep "TIME(irqoff)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni -v "
    fi
    num=$(cat $default_log_path_nosh | grep "TIME(nosched)" -c)
    if test $num -ne 0; then
        echo "Test verbose success."
    else
        handle_error "sysak -g schedmoni -v "
    fi
}

test_threshold() {
    sysak -g schedmoni 1 >/dev/null 2>&1 &
    sleep 5
    pkill schedmoni --signal SIGINT
    sleep 1
    num=$(cat $default_log_path_rqslw | grep "TIME(runslw)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni thresh"
    fi
    num=$(cat $default_log_path_irqof | grep "TIME(irqoff)" -c)
    if test $num -eq 0; then
        handle_error "sysak -g schedmoni thresh"
    fi
    num=$(cat $default_log_path_nosh | grep "TIME(nosched)" -c)
    if test $num -ne 0; then
        echo "Test thresh success."
    else
        handle_error "sysak -g schedmoni thresh"
    fi
}

function test_time() {
    sysak -g schedmoni -s 1 >/dev/null 2>&1 &
    sleep 6

    pid1=`pidof schedmoni`
    if [ $pid1 ]; then
        echo "sysak -g schedmoni -s 1 failed: $pid1"
        handle_failed
    fi

    check_fail
    echo "Test time success."
}

main() {
    test_help
    test_normal
    test_threshold
    test_pid
    test_tid
    test_time
    test_file
    test_json
    test_verbose
    reset
}

main
