#!/bin/sh

temp_log_path="temp.log"

# $sysak -g tracesig --help
# Usage: tracesig [OPTION...]
# Catch the delay of a syscall more than threshold.

# USAGE: tracesig [--help] <[-f a.log]>

# EXAMPLES:
#     tracesig            # run forever, detect delay more than 10ms(default)
#     tracesig -c bash    # check the victim who's name is bash
#     tracesig -f a.log   #save the log to a.log

#   -c, --comm=COMM            Name of the victim
#   -f, --file=FILE            log file
#   -s, --span=SPAN            How long to run
#   -v, --verbose              Verbose debug output
#   -?, --help                 Give this help list
#       --usage                Give a short usage message
#   -V, --version              Print program version

# Mandatory or optional arguments to long options are also mandatory or optional
# for any corresponding short options.

# tracesig
# tracesig -c stress
# tarcesig -v
# tracesig -v -f log
# tracesig -v -s 10 -f log

reset() {
    rm -rf $temp_log_path
}

# tracesig --help
test_help() {
    reset
    val=$(sysak -g tracesig --help | grep 'USAGE: tracesig' -c)

    if test $val -eq 1; then
        echo "Test help success."
    else
        echo "Test help failed."
        exit 1
    fi
}

test_file() {
    reset
    sysak -g tracesig -f $temp_log_path &
    sysak_pid=$!
    sleep 5
    kill $sysak_pid
    num=$(cat $temp_log_path | grep "kill event" -c)
    if test $num -ne 0; then
        echo "Test file success."
    else
        echo "Test file failed."
        exit 1
    fi
}

# tracesig -c stress
test_comm() {
    reset
    sysak -g tracesig -c stress -f $temp_log_path &
    sleep 3 # for setup tracesig
    stress --vm 1 &
    pkill stress
    killall tracesig

    event_num=$(cat $temp_log_path | grep "kill event" -c)
    stress_num=$(cat $temp_log_path | grep "stress" -c)

    if test $event_num -eq $stress_num; then
        echo "Test comm success."
    else
        echo "Test comm failed."
        exit 1
    fi
}

test_span() {
    reset
    killall tracesig
    sysak -g tracesig 5 >/dev/null 2>&1 &
    # echo "start sleep"
    sleep 15
    # echo "sleep 15s done"
    ps aux | grep tracesig -c > $temp_log_path
    num=$(cat $temp_log_path)
    # for tracesig-test.sh and grep
    if test $num -eq 2; then
        echo "Test span success."
    else
        echo "Test span failed."
        exit 1
    fi
}

main() {
    test_help
    test_file
    test_comm
    test_span
}

main
