#!/bin/sh

#Usage: sysak workqlatency [options] [args]
#Options:
#    --threshold/-l		specify the threshold time(ms), default=10ms
#    --help/-h			help info
#example: sysak workqlatency -t 10  #trace work runtime and latency statistics
latency_file="/var/log/sysak/workqlatency/latency.log"
runtime_file="/var/log/sysak/workqlatency/runtime.log"
temp_log_path="workqlatency_tmp.log"

function clean() {
    rm $temp_log_path
}

function handle_failed() {
    echo "Test workqlatency failed." 
    clean
    exit 1
}

function test_help() {
    sysak -g workqlatency -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: sysak workqlatency" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test workqlatency helpinfo success."
    fi
}

function test_workqlatency() {
    yum install -y stress
    sysak workqlatency -l 10 $temp_log_path 2>&1 &
    stress --cpu 1 &
    sleep 5

    killall stress
    killall sysak
    cat $latency_file | grep "Max delay start(s)" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test workqlatency latency success."
    fi

    cat $runtime_file | grep "Max runtime start(s)" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test workqlatency runtime success."
    fi

}

function main() {
    test_help
    test_workqlatency
    clean
}

main
