#!/bin/sh

#Usage: kvmexittime [OPTION...]
#Trace kvm exit time.
#USAGE: kvmexittime [--help] [-p PID] [-t TID] [interval]
#EXAMPLES:
#    kvmexittime        # trace whole system kvm exit time
#    kvmexittime 5      # trace 5 seconds summaries
#    kvmexittime -p 123 # trace kvm exit time for pid 123
#    kvmexittime -t 123 # trace kvm exit time for tid 123 (use for threads only)
#  -p, --pid=PID              Process PID to trace
#  -t, --tid=TID              Thread ID to trace
#  -v, --verbose              Verbose debug output
#  -?, --help                 Give this help list
#      --usage                Give a short usage message
# -V, --version              Print program version

temp_log_path="kvmexittime_tmp.log"

function clean() {
    rm $temp_log_path
    rm $test_file
}

function handle_failed() {
    echo "Test kvmexittime failed." 
    clean
    exit 1
}

function test_help() {
    sysak -g kvmexittime -h > $temp_log_path 2>&1 && cat $temp_log_path | grep "Usage: kvmexittime" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test kvmexittime helpinfo success."
    fi
}

function test_kvmexittime_no_args() {
    sysak kvmexittime > $temp_log_path 2>&1 && cat $temp_log_path | grep "xxxx" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak kvmexittime success."
    fi
}

function test_kvmexittime_p() {
    sysak kvmexittime -p 123 > $temp_log_path 2>&1 && cat $temp_log_path | grep "xxxxx" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak kvmexittime -p 1 success."
    fi
}

function test_kvmexittime_t() {
    sysak kvmexittime -t 123 > $temp_log_path 2>&1 && cat $temp_log_path | grep "xxxxx" > /dev/null
    if test $? -ne 0
    then
        handle_failed
    else
        echo "Test sysak kvmexittime -t 1 success."
    fi
}

function main() {
    test_help
    test_kvmexittime_no_args
    test_kvmexittime_p
    test_kvmexittime_t
    clean
}

main
